(* $MirBSD: src/gnu/gcc/gcc/p/units/md5.pas,v 1.3 2004/04/20 18:26:18 tg Exp $
 *-
 * Copyright (c) 2004
 *	Thorsten "mirabile" Glaser <x86@ePost.de>
 * Copyright (c) 1995, 1996, 2000-2003
 *	Free Software Foundation, Inc. <gpc@gnu.de>
 *	written by Frank Heckenbach
 *
 * Subject to these terms, everybody who obtained a copy of this work
 * is hereby permitted to deal in the work without restriction inclu-
 * ding without limitation the rights to use, distribute, sell, modi-
 * fy, publically perform, give away, merge or sublicence it provided
 * this notice is kept and the authors and contributors are given due
 * credit in derivates or accompanying documents.
 *
 * This work is provided by its developers (authors and contributors)
 * "as is" and without any warranties whatsoever, express or implied,
 * to the maximum extent permitted by applicable law; in no event may
 * developers be held liable for damage caused, directly or indirect-
 * ly, but not by a developer's malice intent, even if advised of the
 * possibility of such damage.
 *-
 * Replacement for the GNU md5 unit, which was licenced under the GNU
 * GPL, thus propagating the requirement to be GPL'd to any code that
 * used the old unit. The GNU md5 unit was the only one not under the
 * LGPL or GPL with library exception clause; this fixes it.
 *
 * The original author of the unit accepted to place the remainder of
 * this code and framework below under this licence shown above, with
 * the only condition that the "md5c.c" file being used is compatible
 * to the GNU GPL (as defined by the FSF).
 *)

{$gnu-pascal,I-}
{$if __GPC_RELEASE__ < 20030303}
{$error This unit requires GPC release 20030303 or newer.}
{$endif}

unit MD5;

interface

uses GPC;

{ Representation of a MD5 value. It is always in little endian byte
  order and therefore portable. }
type
  Card8 = Cardinal attribute (Size = 8);
  TMD5 = array [1 .. 16] of Card8;

const
  MD5StrLength = 2 * High (TMD5);

type
  MD5String = String (MD5StrLength);

{ Computes MD5 message digest for Length bytes in Buffer. }
procedure MD5Buffer (const Buffer; Length: SizeType; var MD5: TMD5); attribute (name = '_p_MD5Buffer');

{ Computes MD5 message digest for the contents of the file f. }
procedure MD5File (var f: File; var MD5: TMD5); attribute (iocritical, name = '_p_MD5File');

{ Initializes a MD5 value with zeros. }
procedure MD5Clear (var MD5: TMD5); attribute (name = '_p_MD5Clear');

{ Compares two MD5 values for equality. }
function MD5Compare (const Value1, Value2: TMD5): Boolean; attribute (name = '_p_MD5Compare');

{ Converts an MD5 value to a string. }
function MD5Str (const MD5: TMD5) = s: MD5String; attribute (name = '_p_MD5Str');

{ Converts a string to an MD5 value. Returns True if successful. }
function MD5Val (const s: String; var MD5: TMD5): Boolean; attribute (name = '_p_MD5Val');

{ Composes two MD5 values to a single one. }
function MD5Compose (const Value1, Value2: TMD5) = Dest: TMD5; attribute (name = '_p_MD5Compose');


implementation

type
  Card32 = Cardinal attribute (Size = 32);
  Card64 = Cardinal attribute (Size = 64);
  TCtx = record { = MD5_CTX }
    state: array[0..3] of Card32;
    count: Card64;
    buffer: array[0..63] of Card8;
  end;

{$L md5c.c}
procedure cInit(var ctx: TCtx);
  external name 'MD5Init';
procedure cUpdate(var ctx: TCtx; protected var buf; len: SizeType);
  external name 'MD5Update';
procedure cFinal(var buf: TMD5; var ctx: TCtx);
  external name 'MD5Final';


procedure MD5Buffer (const Buffer; Length: SizeType; var MD5: TMD5);
var Ctx: TCtx;
begin
  cInit (Ctx);
  cUpdate (Ctx, Buffer, Length);
  cFinal (MD5, Ctx);
end;

procedure MD5File (var f: File; var MD5: TMD5);
var
  Ctx: TCtx;
  Buffer: array [1 .. 4096] of Card8;
  BytesRead: SizeType;
begin
  cInit (Ctx);
  Reset (f, 1);
  repeat
    BlockRead (f, Buffer, SizeOf (Buffer), BytesRead);
    if InOutRes <> 0 then Exit;
    cUpdate (Ctx, Buffer, BytesRead);
  until EOF (f);
  cFinal (MD5, Ctx);
end;

procedure MD5Clear (var MD5: TMD5);
var i: Integer;
begin
  for i := Low (MD5) to High (MD5) do MD5[i] := 0
end;

function MD5Compare (const Value1, Value2: TMD5): Boolean;
var i: Integer;
begin
  MD5Compare := False;
  for i := Low (Value1) to High (Value1) do
    if Value1[i] <> Value2[i] then Exit;
  MD5Compare := True
end;

function MD5Str (const MD5: TMD5) = s: MD5String;
var i: Integer;
begin
  s := '';
  for i := Low (MD5) to High (MD5) do
    s := s + NumericBaseDigits[MD5[i] div $10] + NumericBaseDigits[MD5[i] mod $10]
end;

function MD5Val (const s: String; var MD5: TMD5): Boolean;
var i, d1, d2: Integer;

  function Char2Digit (ch: Char): Integer;
  begin
    case ch of
      '0' .. '9': Char2Digit := Ord (ch) - Ord ('0');
      'A' .. 'Z': Char2Digit := Ord (ch) - Ord ('A') + $a;
      'a' .. 'z': Char2Digit := Ord (ch) - Ord ('a') + $a;
      else        Char2Digit := -1
    end
  end;

begin
  MD5Val := False;
  if Length (s) <> 2 * (High (MD5) - Low (MD5) + 1) then Exit;
  for i := Low (MD5) to High (MD5) do
    begin
      d1 := Char2Digit (s[2 * (i - Low (MD5)) + 1]);
      d2 := Char2Digit (s[2 * (i - Low (MD5)) + 2]);
      if (d1 < 0) or (d1 >= $10) or (d2 < 0) or (d2 >= $10) then Exit;
      MD5[i] := $10 * d1 + d2
    end;
  MD5Val := True
end;

function MD5Compose (const Value1, Value2: TMD5) = Dest: TMD5;
var Buffer: array [1 .. 2] of TMD5;
begin
  Buffer[1] := Value1;
  Buffer[2] := Value2;
  MD5Buffer (Buffer, SizeOf (Buffer), Dest)
end;

end.
